////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : EnumDonglesDemo
//  * Purpose   :    TEnumDonglesEx
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2012.
//  * Version   : 1.0
//  * Home Page : http://rouse.drkb.ru
//  * Home Blog : http://alexander-bagel.blogspot.com/
//  ****************************************************************************
//

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, GrdEnumDonges, StdCtrls, Menus;

type
  TfrmMain = class(TForm)
    memLog: TMemo;
    MainMenu: TMainMenu;
    mnuFile: TMenuItem;
    mnuRefresh: TMenuItem;
    mnuSeparator: TMenuItem;
    mnuExit: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure mnuRefreshClick(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
  private
    Dongles: TEnumDonglesEx;
  protected
    procedure AppendDongleInfo(Value: TDongleQueryRecord;
      const DevicePath: string);
    procedure EraseLog;
    procedure ShowPresentDongles;
    procedure ShowGuardantFlashDrives;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.AppendDongleInfo(Value: TDongleQueryRecord;
  const DevicePath: string);

  function PublicToStr(Value: DWORD): string;
  var
    I: Integer;
    Tmp: DWORD;
  begin
    Result := '';
    for I := 0 to 5 do
    begin
      Tmp := Value mod $28;
      Value := (Value - Tmp) div $28;
      case Tmp of
        0..9: Tmp := Ord(IntToStr(Tmp)[1]);
        10..35: Inc(Tmp, $37);
      else
        Continue;
      end;
      Result := AnsiChar(Tmp) + Result;
    end;
  end;

  function GetDate(Value: DWORD): String;
  //    year    month   day
  // (1111111)(1 111) (11111) 11111111 11111111
  begin
    Result := Format('%.2d.%.2d.%d', [
      Byte(Value shr 16) and $1F,
      (Word(Value shr 16) shr 5) and $F,
      (Byte(Value shr 24) shr 1) and $7F + 1990]);
  end;

  function IsDonglePresent(const Value: string): Boolean;
  begin
    Result := Pos('NTPNP_PCI', Value) > 0;
    if not Result then
      Result := Pos('USBPDO', Value) > 0;
  end;

begin
  memLog.Lines.Add('Found Dongle.');
  memLog.Lines.Add(Format('ID: %.8xh (%dd)',
    [Value.dwID, Value.dwID]));
  memLog.Lines.Add(Format('Public code: %d (%s)',
    [Value.dwPublicCode, PublicToStr(Value.dwPublicCode)]));
  memLog.Lines.Add(Format('Create Date: %s', [GetDate(Value.dwID)]));
  memLog.Lines.Add(Format('Version: %d, SN: %d', [Value.byVer, Value.wSN]));
  memLog.Lines.Add(Format('Device Path: %s', [DevicePath]));
  if IsDonglePresent(DevicePath) then
    memLog.Lines.Add('Dongle checked.')
  else
    memLog.Lines.Add('Invalid dongle, possible emulation.');
  memLog.Lines.Add('');
end;

procedure TfrmMain.EraseLog;
begin
  memLog.Lines.Clear;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Dongles := TEnumDonglesEx.Create;
  ShowPresentDongles;
end;

procedure TfrmMain.ShowGuardantFlashDrives;
var
  I: Integer;
begin
  if Dongles.GuardantFlashDrives.Count = 0 then
    memLog.Lines.Add('Guardant flash drives not present.')
  else
  begin
    memLog.Lines.Add('Current Guardant flash drives:');
    for I := 0 to Dongles.GuardantFlashDrives.Count - 1 do
      memLog.Lines.Add(Format('- %s:\', [Dongles.GuardantFlashDrives[I]]));
  end;
end;

procedure TfrmMain.ShowPresentDongles;
var
  I: Integer;
begin
  EraseLog;
  Dongles.Update;
  for I := 0 to Dongles.Count - 1 do
    AppendDongleInfo(Dongles.Item[I].Data, Dongles.Item[I].PnPParentPath);
  ShowGuardantFlashDrives;
end;

procedure TfrmMain.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.mnuRefreshClick(Sender: TObject);
begin
  ShowPresentDongles;
end;

end.
